mod canvas;
mod input;
mod rdp_ws;
mod utils;

use rdp_ws::Rdp;
use tracing_wasm::WASMLayerConfigBuilder;
use wasm_bindgen::prelude::*;
use wasm_bindgen_futures::spawn_local;

#[wasm_bindgen]
extern "C" {
    pub fn setClipBoard(s: String);
    pub fn getClipBoard() -> String;
    fn wrongCredential();
    pub fn disconnectedAndRefresh(msg: Option<&str>);
    fn jsExportedCredentials(key: &str) -> String;
}

fn start_websocket() -> Result<(), JsValue> {
    // connect
    let url = jsExportedCredentials("wsurl");

    spawn_local(async move {
        let username = jsExportedCredentials("user");
        let password = jsExportedCredentials("password");
        let domain = jsExportedCredentials("domain");
        let performance_flags: u32 = jsExportedCredentials("performanceFlags").parse().unwrap();
        let mut rdp = Rdp::new(&url, &username, &password, &domain, performance_flags);
        if !rdp.start().await {
            wrongCredential();
        }
        rdp.main_loop().await
    });

    Ok(())
}

#[wasm_bindgen(start)]
pub fn run_app() -> Result<(), JsValue> {
    utils::set_panic_hook();
    tracing_wasm::set_as_global_default_with_config(
        WASMLayerConfigBuilder::new()
            .set_max_level(tracing::Level::INFO)
            .build(),
    );
    start_websocket()
}
